﻿/*
 * Author: .net follower (http://dotnetfollower.com) 
 * Title: AdjustAssociationData
 * Copyright: © 2012 .net follower 
 * Usage: software is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 */

using System;
using System.Xml;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Workflow;

namespace AdjustAssociationData
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                AppArguments arguments = new AppArguments(args);

                if(string.IsNullOrEmpty(arguments.Url))
                {
                    Console.WriteLine("Invalid application URL!");
                    return;
                }

                using (SPSite spSite = new SPSite(arguments.Url))
                    using (SPWeb spWeb = spSite.OpenWeb())
                        AdjustAllWorkflowAssociations(spWeb);

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// Adjusts AssociationData properties of all Workflow Associations related to the passed Web, its child Lists and Content Types
        /// </summary>
        public static void AdjustAllWorkflowAssociations(SPWeb web)
        {
            AdjustWebWorkflowAssociation(web);

            for (int i = 0; i < web.ContentTypes.Count; i ++)
                AdjustContentTypeWorkflowAssociation(web.ContentTypes[i]);

            for (int i = 0; i < web.Lists.Count; i ++)
            {
                AdjustListWorkflowAssociation(web.Lists[i]);
                for (int j = 0; j < web.Lists[i].ContentTypes.Count; j ++)
                    AdjustContentTypeWorkflowAssociation(web.Lists[i].ContentTypes[j]);
            }
        }

        /// <summary>
        /// Adjusts AssociationData properties of all Workflow Associations related to the passed List
        /// </summary>
        public static void AdjustListWorkflowAssociation(SPList list)
        {
            AdjustAssociationData(list.WorkflowAssociations);
        }

        /// <summary>
        /// Adjusts AssociationData properties of all Workflow Associations related to the passed Web
        /// </summary>
        public static void AdjustWebWorkflowAssociation(SPWeb web)
        {
            AdjustAssociationData(web.WorkflowAssociations);
        }

        /// <summary>
        /// Adjusts AssociationData properties of all Workflow Associations related to the passed Content Type
        /// </summary>
        public static void AdjustContentTypeWorkflowAssociation(SPContentType contentType)
        {
            AdjustAssociationData(contentType.WorkflowAssociations);
        }

        /// <summary>
        /// Adjusts AssociationData properties of all Workflow Associations in the passed collection
        /// </summary>
        public static void AdjustAssociationData(SPWorkflowAssociationCollection collection)
        {
            for (int i = 0; i < collection.Count; i ++)
                AdjustAssociationData(collection[i], collection);
        }

        /// <summary>
        /// Sets AssociationData property if it's not valid
        /// </summary>
        public static void AdjustAssociationData(SPWorkflowAssociation workflowAssociation, SPWorkflowAssociationCollection collection)
        {
            if (!IsValidXml(workflowAssociation.AssociationData))
            //if (workflowAssociation.AssociationData.StartsWith("<Data"))
            {
                string newValue = string.IsNullOrEmpty(workflowAssociation.AssociationData)
                                      ? "<Data />"
                                      : string.Format("<Data>{0}</Data>", workflowAssociation.AssociationData);
                workflowAssociation.AssociationData = newValue;
                collection.Update(workflowAssociation);
            }
        }

        /// <summary>
        /// Checks if the passed string is a valid xml
        /// </summary>
        public static bool IsValidXml(string str)
        {
            if (!string.IsNullOrEmpty(str))
            {
                try
                {
                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.LoadXml(str);
                    return true;
                }
                catch {}
            }
            return false;
        }
    }
}
